import pandas as pd
import numpy as np


class MFI:
    """
    This class calculate the money flow index for default 14-days period .
    ...
    Attributes
    ----------
    close : pd.Series
    low : pd.Series
    high : pd.Series
    volume: pd.Series
    Methods
    -------
    typical_price(high,low,close):
        Returns the typical price.

    raw_money_flow(volume,typical_price):
        Returns the raw money flow.

    up_or_down(self):
        Returns 1.0 or -1.0 which in the list.

    one_period_positive_mf(self):
        Returns the positive money flow for 1-period.

    one_period_negative_mf(self):
        Returns the negative money flow for 1-period.

    positive_money_flow(period:int,data):
        Returns the positive money flow.

    negative_money_flow(period:int,data):
        Returns the negative money flow.

    money_ratio(positive_money_flow, negative_money_flow,period):
        Returns the money ratio.

    money_flow_index(money_ratio):
        Return the money flow index (mfi)
    """

    def __init__(
        self,
        close: pd.Series,
        low: pd.Series,
        high: pd.Series,
        volume: pd.Series,
        period: int = 14,
    ):
        self.close = close
        self.low = low
        self.high = high
        self.volume = volume
        self.period = period

    def typical_price(self) -> pd.Series:
        return (self.high + self.low + self.close) / 3

    def raw_money_flow(self) -> pd.Series:
        return self.volume * self.typical_price()[1:]

    def up_or_down(self) -> list:
        up_or_down = [np.nan]
        typical_price = self.typical_price()
        close_list = self.close.tolist()

        for i in range(len(close_list) - 1):
            if typical_price[i] < typical_price[i + 1]:
                up_or_down.append(1.0)
            else:
                up_or_down.append(-1.0)
        return up_or_down

    def one_period_positive_mf(self) -> pd.Series:
        up_or_down = self.up_or_down()
        one_period_positive_mf = [np.nan]
        raw_money_flow = self.raw_money_flow()
        for i in range(1, len(up_or_down)):
            if up_or_down[i] == np.nan:
                continue
            elif up_or_down[i] == 1.0:
                one_period_positive_mf.append(raw_money_flow[i])
            else:
                one_period_positive_mf.append(0)
        return pd.Series(one_period_positive_mf)

    def one_period_negative_mf(self) -> pd.Series:
        up_or_down = self.up_or_down()
        one_period_negative_mf = [np.nan]
        raw_money_flow = self.raw_money_flow()
        for i in range(1, len(up_or_down)):
            if up_or_down[i] == np.nan:
                continue
            elif up_or_down[i] == -1.0:
                one_period_negative_mf.append(raw_money_flow[i])
            else:
                one_period_negative_mf.append(0)
        return pd.Series(one_period_negative_mf)

    def positive_money_flow(self) -> pd.Series:
        positive_mf = self.one_period_positive_mf().rolling(window=self.period).sum()
        return positive_mf

    def negative_money_flow(self) -> pd.Series:
        negative_mf = self.one_period_negative_mf().rolling(window=self.period).sum()
        return negative_mf

    def money_ratio(self) -> pd.Series:
        return self.positive_money_flow() / self.negative_money_flow()

    def money_flow_index(self) -> pd.Series:
        mfi = 100 - (100 / (1 + self.money_ratio()))
        return mfi



import pandas as pd     
 
class SMA:
    def __init__(self,close,period=5):
        self.df=pd.DataFrame(data={"close":close})
        self.period=period

    def Sma(self):
        # calculate given period moving average using Pandas
        self.df['sma'] = self.df['close'].rolling(self.period).mean()
        return self.df['sma']



import pandas as pd
import numpy as np


class ParabolicSAR:

    """
    This class calculates ParobolicSAR. All initials connected each other.
    SAR means stop and reverse.
    If PSAR point is lower than the actual data it means "BUY" else it means "SELL".

    Attributes
    ---------------
    high : list
    low : list
    close : list
    """

    def __init__(self, high: list, low: list, close: list):
        self.high = high
        self.low = low
        self.close = close

    def parabolic_sar(self) -> list:

        # first, setting up initial values

        # ep is a list of extremum points
        ep = [self.low[0]]

        # psar is a list of parabolic stop and reverse values
        psar = [self.high[0]]

        # acc point is a accumulation point list(maximum accumulation point is 0.2)
        acc_point = [0.02]

        # trend is a list of trend of graphic(falling and rising)
        trend = ["Falling"]

        # calc_psar is a list of calculation of below function
        calc_psar = [(psar[0] - ep[0]) * acc_point[0]]

        # initial_psar is a list of values determines actual psar
        initial_psar = []

        for i in range(len(self.high) - 1):
            # setting initial psar values according to trend
            if trend[i] == "Falling":
                try:
                    initial_psar.append(
                        max(
                            (psar[i] - calc_psar[i]),
                            np.array(self.high[i - 1 : i + 1]).max(),
                        )
                    )
                except ValueError:
                    initial_psar.append(
                        max((psar[i] - calc_psar[i]), np.array(self.high[0:1]).max())
                    )
            elif trend[i] == "Rising":
                try:
                    initial_psar.append(
                        min(
                            (psar[i] - calc_psar[i]),
                            np.array(self.low[i - 1 : i + 1]).min(),
                        )
                    )
                except ValueError:
                    initial_psar.append(
                        max((psar[i] - calc_psar[i]), np.array(self.low[0:1]).min())
                    )

            # setting psar values according to trend,initial_psar and given data
            if trend[i] == "Falling" and self.high[i + 1] < initial_psar[i]:
                psar.append(initial_psar[i])
            elif trend[i] == "Falling" and self.high[i + 1] >= initial_psar[i]:
                psar.append(ep[i])
            elif trend[i] == "Rising" and self.low[i + 1] > initial_psar[i]:
                psar.append(initial_psar[i])
            elif trend[i] == "Rising" and self.low[i + 1] <= initial_psar[i]:
                psar.append(ep[i])

            # decide on the trend according to psar and close data
            if psar[i + 1] > self.close[i + 1]:
                trend.append("Falling")
            else:
                trend.append("Rising")

            # setting extremum point
            if trend[i + 1] == "Falling":
                ep.append(min(ep[i], self.low[i + 1]))
            else:
                ep.append(max(ep[i], self.high[i + 1]))

            # decide on the accumulation point
            if trend[i] == trend[i + 1] and ep[i] != ep[i + 1] and acc_point[i] < 0.2:
                acc_point.append(0.02 + acc_point[i])
            elif trend[i] == trend[i + 1] and ep[i] == ep[i + 1]:
                acc_point.append(acc_point[i])
            elif trend[i] != trend[i + 1]:
                acc_point.append(acc_point[0])
            else:
                acc_point.append(0.2)

            # decide on the result of ((psar-ep)*acc_point)
            calc_psar.append((psar[i + 1] - ep[i + 1]) * acc_point[i + 1])

        return psar



import numpy as np
import pandas as pd

class CMO:
    '''
    This class calculate CMO for default 10-days period.

    It needs to get 'close' values and period(optional).
    
    '''
    def __init__(self,close:list,period:int=10) -> None:
        self.period = period
        self.close = close
    
    '''
    From period, returns a list
    by using cumulative function.
    Sum of the difference between the current close
    and previous close on up days for the specified period. 
    Up days are days when the current close is greater than the previous close.
    '''
    def cumulativeUp(self):
        up = [0]
        retUp = np.full(10,np.nan)
        for i in range(1,len(self.close)):
            if self.close[i]>self.close[i-1]:
                upT = self.close[i] - self.close[i-1]
                up.append(upT)
            else:
                up.append(0)

        cumulativeUp = np.cumsum(up)

        for i in range(0,len(cumulativeUp)-self.period):
            retUp = np.append(retUp,cumulativeUp[i+self.period]-cumulativeUp[i])

        return np.array(retUp)
    '''
    From period, returns a list
    by using cumulative function.
    Sum of the absolute value of the difference between the current close 
    and the previous close on down days for the specified period.
    Down days are days when the current close is less than the previous close.
    '''    
    
    def cumulativeDown(self):
        down = np.zeros(1)
        retDown = np.full(10,np.nan)
        for i in range(1,len(self.close)):
            if self.close[i]<self.close[i-1]:
                downT = abs(self.close[i]-self.close[i-1])
                down = np.append(down,downT)
            else:
                down = np.append(down,0)

        cumulativeDown = np.cumsum(down)

        for i in range(0,len(cumulativeDown)-self.period):
            retDown = np.append(retDown,cumulativeDown[i+self.period]-cumulativeDown[i])
        
        return retDown

    '''
    Running function for the class.
    '''
    def cmo(self) -> np.ndarray:
        
        retUp = self.cumulativeUp()
        retDown = self.cumulativeDown()
        cmo = ((retUp-retDown)/(retDown+retUp))*100
        return cmo


import pandas as pd


class IchimokuCloud:
    """
    The best settings for Ichimoku 5 min chart is standard default settings 9-26-52,
    where 9 is for the Conversion Line ( Tenkan-sen), 26 for the Base Line (Kyun-sen ), and 52 for the Leading Span B (Senkou Span B).
    The best Ichimoku settings for scalping and 1-minute chart are 12-24-120,
    where 12 is for the Conversion Line ( Tenkan-sen), 24 for the Base Line (Kyun-sen ), and 120 for the Leading Span B (Senkou Span B).
    """

    def __init__(
        self,
        closes_list: list,
        lowest_list: list,
        higest_list: list,
        period_list: list = [9, 26, 52, 26],
    ):
        self.lowest_list = lowest_list
        self.higest_list = higest_list
        self.closes_list = closes_list
        self.ts_period = period_list[0]
        self.ks_period = period_list[1]
        self.ssb_period = period_list[2]
        self.cs_period = period_list[3]

    def __founder_higest_price(self, day):
        # This function find the higest price in the past
        return float(max(self.higest_list[-1 : -(day + 1) : -1]))

    def __founder_lowest_price(self, day):
        # This function find the lowest price in the past
        return float(min(self.lowest_list[-1 : -(day + 1) : -1]))

    def __tenkan_sen(self) -> float:
        # the formula of Tenkan Sen => the sum of the higest and lowest price of last 9 days is divied by 2.
        tenkan_sen = (
            self.__founder_higest_price(self.ts_period)
            + self.__founder_lowest_price(self.ts_period)
        ) / 2
        return tenkan_sen

    def __kijun_sen(self) -> float:
        # the formula of Kijun Sen => the sum of the higest and lowest price of last 26 days is divied by 2.
        kijun_sen = (
            self.__founder_higest_price(self.ks_period)
            + self.__founder_lowest_price(self.ks_period)
        ) / 2
        return kijun_sen

    def __senkou_span_A(self) -> float:
        # the formula of Senkou Span A => the sum of Tenkan Sen and Kijun Sen is divied by 2.
        senkou_span_A = (self.__tenkan_sen() + self.__kijun_sen()) / 2
        return senkou_span_A

    def __senkou_span_B(self) -> float:
        # the formula of Senkou Span B => the sum of the higest and lowest price of last 56 days is divied by 2.
        senkou_span_B = (
            self.__founder_higest_price(self.ssb_period)
            + self.__founder_lowest_price(self.ssb_period)
        ) / 2
        return senkou_span_B

    def __cikou_span(self) -> float:
        # the formula of Cikou Span => Finds the closing value of 26 days ago.
        cikou_span = self.closes_list[-self.cs_period]
        return cikou_span

    def ichimoku_cloud(self) -> pd.Series:
        # this create a pandas series and return.
        functions_name = [
            "Tenkan Sen",
            "Kijun Sen",
            "Senkou Span A",
            "Senkou Span B",
            "Cikou Span",
        ]
        functions_values = [
            self.__tenkan_sen(),
            self.__kijun_sen(),
            self.__senkou_span_A(),
            self.__senkou_span_B(),
            self.__cikou_span(),
        ]
        ichimoku_cloud = pd.Series(functions_values, index=functions_name)

        return ichimoku_cloud



import numpy as np
from numpy import nan as npNaN
import pandas as pd


class EMA:
    """
    This class calculates EMA according to user specified periods.
    If the user does not enter a value,the value is set to 9.

    Methods
    ---------
    sma(self):
        The EMA value cannot be calculated on the days before the period.
        In this function, NaN values are assigned to the days that cannot be calculated.
        The first day calculated is the SMA of the previous days.
    ---
    calc_ema(self):
        Calculates EMA values.
        EMA : (P * a) + (Previous EMA * (1 - a))
            P = Current Price
            a = Smoothing Factor = 2/(1+N)
            N = Period

        Returns pd.series.
    """

    def __init__(self, close: pd.Series, period: int = 9):
        self.close = close
        self.period = period

    def sma(self):
        self.close = self.close.copy()
        sma = self.close[0 : self.period].mean()
        self.close[: self.period - 1] = npNaN
        self.close.iloc[self.period - 1] = sma
        return self.close

    def calc_ema(self) -> pd.Series:
        df = pd.DataFrame()
        self.close = self.sma()
        df["EMA"] = self.close.ewm(span=self.period, adjust=False).mean()
        return df["EMA"]



import pandas as pd


class Aroon:
    """
    This class calculate the aroon function for default 25-days period.
     To calculate this function It needs adj close value.
    """

    def __init__(self, adj_close, period=25) -> None:
        self.period = period
        self.df = pd.DataFrame(data={"Adj Close": adj_close})

    def __high_finder(self) -> pd.Series:
        """
        This method finds the highest data in the period and calculates how many periods
        are there after that the highest data.
        """
        df = (
            self.df["Adj Close"]
            .rolling(self.period + 1)
            .apply(lambda x: x.argmax(), raw=True)
        )
        return (self.period - df + 1).squeeze()

    def __low_finder(self) -> pd.Series:
        """
        This method finds the lowest data in the period and calculates how many periods
        are there after that the lowest data.
        """
        df = (
            self.df["Adj Close"]
            .rolling(self.period + 1)
            .apply(lambda x: x.argmin(), raw=True)
        )
        return (self.period - df + 1).squeeze()

    def aroon_up(self) -> pd.Series:
        """
        In this method , using by __high_finder(self): method and self.period It calculates
        aroon_up function by using 100*((25-self.__high_finder())/25)
        """
        aroon_up = 100 * ((25 - self.__high_finder()) / 25)
        return aroon_up

    def aroon_down(self) -> pd.Series:
        """
         In this method , using by __low_finder(self): method and self.period It calculates
        aroon_down function 100*((25-self.__low_finder())/25)
        """
        aroon_down = 100 * ((25 - self.__low_finder()) / 25)
        return aroon_down

    def aroon_oscilattor(self) -> pd.Series:
        """
        In this method, using aroon_up(self): and aroon_down(self): functions It calculates
        aroon_oscilattor(self):
        """
        return self.aroon_up() - self.aroon_down()



import pandas as pd


class RSI:
    """
    This class calculates RSI for period = 14.
    Attributes
    -----------
    close : pandas
    period : int
    """

    def __init__(self, close, period: int = 14) -> None:
        self.close = close
        self.period = period

    def gain_calculator(self) -> pd.Series:
        """
        From period, returns a pandas series
        by using gain_calculator function.
        Sum of the difference between the current closes
        and previous closes when the current closes
        are less than previous closes.
        Then diveded to 14.
        """
        close = self.close
        gain_list = pd.Series(0.0, index=close.index)
        for i in range(1, len(close)):
            if close[i] > close[i - 1]:
                gain = close[i] - close[i - 1]
                gain_list[i] = gain
        gain_avg = gain_list.rolling(window=self.period).mean()

        return gain_avg

    def loss_calculator(self) -> pd.Series:
        """
        From period, returns a pandas series
        by using loss_calculator function.
        Sum of the difference between the current closes
        and previous closes when the current closes
        are greater than previous closes.
        Then diveded to 14.
        """
        close = self.close
        loss_list = pd.Series(0.0, index=close.index)
        for i in range(1, len(close)):
            if close[i] < close[i - 1]:
                loss = close[i - 1] - close[i]
                loss_list[i] = loss
        loss_avg = loss_list.rolling(window=self.period).mean()

        return loss_avg

    def rsi(self) -> pd.Series:
        """
        In this function, it calculates rs by using gain/loss
        and rsi by using 100-(100/(1+rs)).
        Returns a pandas series.
        """
        gain = self.gain_calculator()
        loss = self.loss_calculator()
        rs = gain / loss
        rs[rs == float("inf")] = 0
        rsi = 100 - (100 / (1 + rs))
        return rsi



class WilliamsR:
    """ "
    #Williams%R say overbought and oversold
    #Williams%R answer is between 0 to -100.
    #If anwer goes above -20, this is overbought. If it drops below -80, this is oversold
    #If answer is above -20 and moving below -20, It's a sign that the price might go down.
    #If answer is below -80 and rising above -80, It's a sign that the price might rise.
    #-10 and -90 reference values are more reliable than -20 and -80

    """

    def __init__(self, current_close: float, lowest_list: list, higest_list: list):
        self.current_close = current_close
        self.lowest_list = lowest_list
        self.higest_list = higest_list

    def __founder_higest_price(self):
        # This function find the higest price of the last 14 days
        return float(max(self.higest_list[-1:-15:-1]))

    def __founder_lowest_price(self):
        # This function find the lowest price of the last 14 days
        return float(min(self.lowest_list[-1:-15:-1]))

    def WILLIAMS_R(self):
        # The formula of Williams%R
        R = (
            (self.__founder_higest_price() - self.current_close)
            / (self.__founder_higest_price() - self.__founder_lowest_price())
            * -100
        )

        return R



import numpy as np
from hyta.movingaverage import MA


class TMA:
    """
    This class calculates TMA for default 20-days period.
    Attributes
    ---------------
    close : list
    period : int (optional, base=20)
    """

    def __init__(self, close, period: int = 20):
        self.period = period
        self.close = close

    """
    Calculating first MA for TMA.
    """

    def ma(self) -> np.ndarray:
        return MA(self.close, self.period).calcAverage()

    """
    Calculating the second MA which equals TMA.
    """

    def tma(self) -> np.ndarray:
        return MA(self.ma(), self.period).calcAverage()



from hyta.ema import EMA
import numpy as np
import pandas as pd


class t3Tillson:
    """
    Calculates the T3 Tillson indicator for a given set of closing prices.
    It uses six exponential moving averages
    calculated using the "EMA" class from the "ema" module.
    The class constructor takes a list or array of
    closing prices as "close" and an optional integer as "period" (default value is 9)
    that specifies the number of periods to use in the calculation.
    """

    def __init__(self, close, period: int = 9) -> None:
        self.period = period
        self.a = 0.618
        self.c1 = -self.a**3
        self.c2 = 3 * (self.a**2) + 3 * (self.a**3)
        self.c3 = -6 * (self.a**2) - 3 * self.a - 3 * (self.a**3)
        self.c4 = 1 + 3 * self.a + self.a**3 + 3 * (self.a**2)
        self.df = pd.DataFrame(data={"Close": close})
        self.close = self.df["Close"]
        self.e1 = self.ema1()
        self.e2 = self.ema2()
        self.e3 = self.ema3()
        self.e4 = self.ema4()
        self.e5 = self.ema5()
        self.e6 = self.ema6()

    def ema1(self) -> pd.Series:
        """
        Calculates ema1 using calc_ema from EMA according to close and period.
        """
        e1 = EMA(self.close, self.period).calc_ema()
        return e1

    def ema2(self) -> pd.Series:
        """
        Calculates ema2 using cals_ema from EMA according to ema1 and period.
        """
        e2 = EMA(self.e1, self.period).calc_ema()
        return e2

    def ema3(self) -> pd.Series:
        """
        Calculates ema3 using cals_ema from EMA according to ema2 and period.
        """
        e3 = EMA(self.e2, self.period).calc_ema()
        return e3

    def ema4(self) -> pd.Series:
        """
        Calculates ema4 using cals_ema from EMA according to ema3 and period.
        """
        e4 = EMA(self.e3, self.period).calc_ema()
        return e4

    def ema5(self) -> pd.Series:
        """
        Calculates ema5 using cals_ema from EMA according to ema4 and period.
        """
        e5 = EMA(self.e4, self.period).calc_ema()
        return e5

    def ema6(self) -> pd.Series:
        """
        Calculates ema6 using cals_ema from EMA according to ema5 and period.
        """
        e6 = EMA(self.e5, self.period).calc_ema()
        return e6

    def T3(self) -> pd.Series:
        """
        Calculates T3 Tillson using the c1*e6 + c2*e5 + c3*e4 + c4*e3 formula
        """
        T3Tillson = (
            self.c1 * self.e6
            + self.c2 * self.e5
            + self.c3 * self.e4
            + self.c4 * self.e3
        )
        return T3Tillson



import pandas as pd

class TSI:
    """
    Class to calculate True Strength Index
    """
    def __init__(self, close:list, long:int=25, short:int=13):
        self.close = close
        self.long = long
        self.short = short
    
    def calc_tsi(self) -> pd.Series :
        df = pd.DataFrame()
        df["close"] = self.close
        df["diff"] = self.close.diff(1)
        df["ema25"] = df["diff"].ewm(span=self.long, min_periods=self.long, adjust=False).mean()
        df["ema13"] = df["ema25"].ewm(span=self.short, min_periods=self.short, adjust=False).mean()
        df["abmo"] = abs(df["diff"])
        df["abmo25"] = df["abmo"].ewm(span=self.long, min_periods=self.long, adjust=False).mean()
        df["abmo13"] = df["abmo25"].ewm(span=self.short, min_periods=self.short, adjust=False).mean()
        df["TSI"] = df["ema13"]/df["abmo13"]*100
        return df["TSI"]




import numpy as np
import pandas as pd
from hyta.atr import ATR


class DX:
    def __init__(
        self, high: pd.Series, low: pd.Series, close: pd.Series, period: int = 14
    ) -> None:
        self.high = high
        self.Low = low
        self.close = close
        self.period = period
        self.dx_data = pd.DataFrame(
            {"High": self.high, "Close": self.close, "Low": self.Low}
        )

    def TR(self) -> pd.Series:
        self.dx_data["TR"] = ATR(
            [self.dx_data["High"], self.dx_data["Low"], self.dx_data["Close"]],
            self.period,
        ).true_range()
        self.dx_data["TR"][0] = np.nan
        return self.dx_data["TR"]

    def plus_dm(self) -> pd.Series:
        first = [np.NAN]

        for i in range(1, len(self.Low)):
            low_diff = self.Low[i - 1] - self.dx_data["Low"][i]
            high_diff = self.dx_data["High"][i] - self.dx_data["High"][i - 1]

            if high_diff > low_diff:
                first.append(max(high_diff, 0))
            else:
                first.append(0)
        self.dx_data["+DM 1"] = pd.DataFrame({"+DM 1": first})
        return self.dx_data["+DM 1"]

    def minus_dm(self) -> pd.Series:
        first = [np.NAN]

        for i in range(1, len(self.dx_data["Low"])):
            low_diff = self.dx_data["Low"][i - 1] - self.dx_data["Low"][i]
            high_diff = self.dx_data["High"][i] - self.dx_data["High"][i - 1]

            if low_diff > high_diff:
                first.append(max(low_diff, 0))
            else:
                first.append(0)
        self.dx_data["-DM 1"] = pd.DataFrame({"-DM 1": first})
        return self.dx_data["-DM 1"]

    def smoothed_plus_dm14(self):
        self.plus_dm()
        first_14_plus_dm = self.dx_data["+DM 1"][: self.period + 1].sum()
        second_14_plus_dm = (
            first_14_plus_dm
            - (first_14_plus_dm / self.period)
            + self.dx_data["+DM 1"][self.period + 1]
        )

        new_dx_Data = np.array([])
        for i in range(self.period):
            new_dx_Data = np.append(new_dx_Data, np.nan)
        new_dx_Data = np.append(new_dx_Data, [first_14_plus_dm, second_14_plus_dm])

        for i in range(len(self.dx_data["+DM 1"]) - 16):
            other_14_plus_dm = (
                new_dx_Data[-1]
                - (new_dx_Data[-1] / self.period)
                + self.dx_data["+DM 1"][i + 16]
            )
            new_dx_Data = np.append(new_dx_Data, other_14_plus_dm)
            self.dx_data["+DM 14"] = pd.DataFrame({"+DM 14": new_dx_Data})
        return self.dx_data["+DM 14"]

    def smoothed_minus_dm14(self):
        self.minus_dm()
        first_14_minus_dm = self.dx_data["-DM 1"][: self.period + 1].sum()
        second_14_minus_dm = (
            first_14_minus_dm
            - (first_14_minus_dm / self.period)
            + self.dx_data["-DM 1"][self.period + 1]
        )

        new_dx_Data = np.array([])
        for i in range(self.period):
            new_dx_Data = np.append(new_dx_Data, np.nan)
        new_dx_Data = np.append(new_dx_Data, [first_14_minus_dm, second_14_minus_dm])

        for i in range(len(self.dx_data["-DM 1"]) - 16):
            other_14_minus_dm14 = (
                new_dx_Data[-1]
                - (new_dx_Data[-1] / self.period)
                + self.dx_data["-DM 1"][i + 16]
            )
            new_dx_Data = np.append(new_dx_Data, other_14_minus_dm14)
            self.dx_data["-DM 14"] = pd.DataFrame({"-DM 14": new_dx_Data})
        return self.dx_data["-DM 14"]

    def smoothed_tr14(self):
        self.TR()
        first_14_tr = self.dx_data["TR"][: self.period + 1].sum()
        second_14_tr = (
            first_14_tr
            - (first_14_tr / self.period)
            + self.dx_data["TR"][self.period + 1]
        )

        new_tr_Data = np.array([])
        for i in range(self.period):
            new_tr_Data = np.append(new_tr_Data, np.nan)
        new_tr_Data = np.append(new_tr_Data, [first_14_tr, second_14_tr])

        for i in range(len(self.dx_data["TR"]) - 16):
            other_14_tr = (
                new_tr_Data[-1]
                - (new_tr_Data[-1] / self.period)
                + self.dx_data["TR"][i + 16]
            )
            new_tr_Data = np.append(new_tr_Data, other_14_tr)
            self.dx_data["Smoothed TR"] = pd.DataFrame({"Smoothed TR": new_tr_Data})
        return self.dx_data["Smoothed TR"]

    def plus_di_14(self):
        self.smoothed_plus_dm14()
        self.smoothed_tr14()
        plus_di_14 = 100 * (self.dx_data["+DM 14"] / self.dx_data["Smoothed TR"])
        return plus_di_14

    def minus_di_14(self):
        self.smoothed_minus_dm14()
        self.smoothed_tr14()
        minus_di_14 = 100 * (self.dx_data["-DM 14"] / self.dx_data["Smoothed TR"])
        return minus_di_14

    def di_diff(self):
        self.plus_di_14()
        self.minus_di_14()
        di_difference = abs(self.plus_di_14() - self.minus_di_14())
        return di_difference

    def di_sum(self):
        self.plus_di_14()
        self.minus_di_14()
        di_sum = self.plus_di_14() + self.minus_di_14()
        return di_sum

    def dx(self):
        self.di_diff()
        self.di_sum()
        dx = abs(100 * (self.di_diff() / self.di_sum()))
        return dx



import numpy as np
import pandas as pd


class UO:
    """
    This class calculate UO for default 28-periods.

    It needs to get 'low','close' and 'high' values respectively.

    """

    def __init__(self, low, close, high) -> None:
        self.high = high
        self.low = low
        self.close = close

    def buyingPressure(self) -> np.ndarray:

        """
        This module, we calculate buying pressure with
        'buying pressure = close - min(low,PriorClose)'
        this equation.
        It needs to get close and low values.
        Returns the BuyingPressure array.
        Type of array is np.ndarray.
        """

        BP = []
        for i in range(0, len(self.close) - 1):
            bp = self.close[i + 1] - min(self.low[i + 1], self.close[i])
            BP.append(bp)
        return np.array(BP)

    def trueRange(self) -> np.ndarray:
        """
        This module, we calculate buying pressure with
        'true range = max(high,PriorClose) - min(low,PriorClose)'
        this equation.
        It needs to get high,close and low values.
        Returns the trueRange array.
        Type of array is np.ndarray.
        """
        TR = []
        for i in range(1, len(self.high)):
            tr = max(self.high[i], self.close[i - 1]) - min(
                self.low[i], self.close[i - 1]
            )
            TR.append(tr)
        return np.array(TR)

    def Average7(self) -> np.ndarray:
        """
        This module, we calculate 7-periods average with
        'A7 = (Sum of 7 period BP) / (Sum of 7 period TR)'
        this equation.
        It needs to get BP and TR values.
        Returns the A7 array.
        Type of array is np.ndarray.
        """
        cumulativeBP = np.cumsum(self.buyingPressure())
        cumulativeBP = np.insert(cumulativeBP, 0, 0)

        sevenPeriodedBP = [
            cumulativeBP[i] - cumulativeBP[i - 7] for i in range(7, len(cumulativeBP))
        ]

        cumulativeTR = np.cumsum(self.trueRange())
        cumulativeTR = np.insert(cumulativeTR, 0, 0)

        sevenPeriodedTR = [
            cumulativeTR[i] - cumulativeTR[i - 7] for i in range(7, len(cumulativeTR))
        ]

        A7 = np.array(sevenPeriodedBP) / np.array(sevenPeriodedTR)

        return A7

    def Average14(self) -> np.ndarray:
        """
        This module, we calculate 14-periods average with
        'A14 = (Sum of 14 period BP) / (Sum of 14 period TR)'
        this equation.
        It needs to get BP and TR values.
        Returns the A14 array.
        Type of array is np.ndarray.
        """
        cumulativeBP = np.cumsum(self.buyingPressure())
        cumulativeBP = np.insert(cumulativeBP, 0, 0)

        fourteenPeriodedBP = [
            cumulativeBP[i] - cumulativeBP[i - 14] for i in range(14, len(cumulativeBP))
        ]

        cumulativeTR = np.cumsum(self.trueRange())
        cumulativeTR = np.insert(cumulativeTR, 0, 0)

        fourteenPeriodedTR = [
            cumulativeTR[i] - cumulativeTR[i - 14] for i in range(14, len(cumulativeTR))
        ]

        A14 = np.array(fourteenPeriodedBP) / np.array(fourteenPeriodedTR)

        return A14

    def Average28(self) -> np.ndarray:
        """
        This module, we calculate 28-periods average with
        'A28 = (Sum of 28 period BP) / (Sum of 28 period TR)'
        this equation.
        It needs to get BP and TR values.
        Returns the A28 array.
        Type of array is np.ndarray.
        """
        cumulativeBP = np.cumsum(self.buyingPressure())
        cumulativeBP = np.insert(cumulativeBP, 0, 0)

        twentyeightPeriodedBP = [
            cumulativeBP[i] - cumulativeBP[i - 28] for i in range(28, len(cumulativeBP))
        ]

        cumulativeTR = np.cumsum(self.trueRange())
        cumulativeTR = np.insert(cumulativeTR, 0, 0)

        twentyeightPeriodedTR = [
            cumulativeTR[i] - cumulativeTR[i - 28] for i in range(28, len(cumulativeTR))
        ]

        A28 = np.array(twentyeightPeriodedBP) / np.array(twentyeightPeriodedTR)

        return A28

    def uo(self) -> pd.Series:
        """
        This module is final module.
        This calculates the UltimateOscillator for 28 periods.
        Needs to get A7,A14,A28 arrays.
        Returns ultOsc with type of pd.Series
        Due to not returns any NaN values, we use index slicing.
        """
        A7 = self.Average7()
        A14 = self.Average14()
        A28 = self.Average28()
        numerator = A7[21:] * 4 + A14[14:] * 2 + A28
        ultOsc = (numerator / 7) * 100
        return pd.Series(ultOsc)



from hyta.wma import WeightedMovingAverage
import math


class HullMovingAverage:
    """ "
    HMAs with shorter periods are often used to identify entry points.
    When the overall trend is up and the HMA turns up, this is a signal to buy long.
    Conversely, when the overall trend is down and the HMA turns down, this is a signal to buy short.
    shoreter period is usually 20.
    """

    """
    Hull Moving Average Formula:
    (A) The First Weighted Moving Average (WMA1):
    WMA1[i] = (Close[i − N + 1] + 2 × Close[i − N + 2] + 3 × Close[i − N + 3] + … + N × Close[i]) / (N × (N + 1) × 0.5) where:
    N = Hull Moving Average look back;
    Index: i ~ Current Bar.
    (B) The Second Weighted Moving Average (WMA2):
    WMA2[i] = (Close[i − M + 1] + 2 × Close[i − M + 2] + 3 × Close[i − M + 3] + … + M × Close[i]) / (M × (M + 1) × 0.5) where:
    M = round(N/2);
    Index: i ~ Current Bar.
    (C) The Hull Moving Average (HMA):
    Delta[i] = 2 × WMA2[i] − WMA1[i];
    HMA[i] = (Delta[i − K + 1] + 2 × Delta[i − K + 2] + 3 × Delta[i − K + 3] + … + K × Delta[i])/(K × (K + 1) × 0.5) where:
    K = round(SquareRoot(N));
    Index: i ~ Current Bar.
    """

    def __init__(self, close_list, period):
        self.close_list = close_list
        self.period = period
        self.wma = WeightedMovingAverage

    def __number_rounder(self, num) -> int:
        # this function round number
        return int(num + 0.5)

    def __wma1(self, number) -> float:
        # Returns the first weighted moving average, using half the period as the length of the moving average

        return self.wma(
            self.close_list[: -number or None], self.__number_rounder(self.period / 2)
        ).weighted_moving_average()

    def __wma2(self, number) -> float:
        # Returns the second weighted moving average, using the period as the length of the moving average

        return self.wma(
            self.close_list[: -number or None], self.period
        ).weighted_moving_average()

    def __raw_hma(self, number) -> float:
        # Returns the raw Hull Moving Average, which is calculated as (2 * wma1) - wma2
        return (2 * self.__wma1(number)) - self.__wma2(number)

    def raw_hma_list(self) -> list:
        raw_hma_list = []
        for i in range(self.__number_rounder(math.sqrt(self.period)), -1, -1):
            raw_hma_list.append(self.__raw_hma(i))

        return raw_hma_list

    def hull_moving_average(self) -> float:
        # Returns the final Hull Moving Average, which is calculated using a weighted moving average of the raw HMAs
        hma = self.wma(
            self.raw_hma_list(), self.__number_rounder(math.sqrt(self.period))
        ).weighted_moving_average()

        return hma



import numpy as np
import pandas as pd


class KAMA:
    """
    This class calculates Kaufman's Adaptive Moving Average.

    Methods
    --------------------
    change(self):
        Calculates the change needed to find efficiency ratio.
        change = abs(Close - Close(10 periods ago))
    -----
    volatility(self):
        Calculates the volatility needed to find efficiency ratio.
        Volatility is the sum of the absolute value of the last ten price changes.

        volatility = sum10(abs(Close - Prior Close))
    -----
    efficiency_ratio(self):
        This method calculates efficiency ratio.
        The efficiency ratio shows the efficiency of price changes.

        ER = Change/Volatility
    -----
    smoothing_factor(self):
        The smoothing constant uses the ER and two smoothing constants
        based on an exponential moving average.

        SC = [ER * (fastest SC - slowest SC) + slowest SC] ** 2
    -----
    calc_kama(self):
        Calculates the KAMA values.

        CurrentKAMA = PriorKama + SC * (CurrentPrice - PriorKAMA)

        Returns pd.Series.
    """

    def __init__(
        self, close: pd.Series, period: int = 10, fastest: int = 2, slowest: int = 30
    ):
        self.close = close
        self.period = period
        self.fastest = fastest
        self.slowest = slowest
        self.df = pd.DataFrame({"Close": self.close})

    def change(self):
        self.df["Change"] = abs(self.df["Close"].diff(periods=10))
        return self.df["Change"]

    def volatility(self):
        self.df["Volatility"] = abs(self.df["Close"].diff(periods=1))
        return self.df["Volatility"]

    def efficiency_ratio(self):
        self.change()
        self.volatility()
        self.df["ER"] = (
            self.df["Change"] / self.df["Volatility"].rolling(window=self.period).sum()
        )
        return self.df["ER"]

    def smoothing_constant(self):
        self.efficiency_ratio()
        self.df["sc"] = (
            (self.df["ER"] * (2 / (self.fastest + 1) - 2 / (self.slowest + 1)))
            + (2 / (self.slowest + 1))
        ) ** 2
        return self.df["sc"]

    def calc_kama(self):
        self.smoothing_constant()
        self.df["kama"] = pd.Series(dtype="float64")

        for i in range(1, len(self.df["Close"])):
            self.df["kama"][: self.period - 1] = np.nan
            self.df["kama"].iloc[self.period - 1] = self.df["Close"].iloc[
                self.period - 1
            ]

            if self.df["kama"][i] != np.nan:
                self.df["kama"][i] = self.df["kama"][i - 1] + self.df["sc"][i] * (
                    self.df["Close"][i] - self.df["kama"][i - 1]
                )
        return self.df["kama"]



import pandas as pd
import numpy as np


class ATR:
    """
    This class calculates ATR for default True_Value=14.
    True_Value must be positive integer.
    data_all=np.array([data_high,data_low,data_close]) data comes with this format.

    Attributes
    ---------------
    data_all  : np.array
    True_Value: int
    """

    def __init__(self, data_all: np.ndarray, true_value: int = 14):
        self.data_all = data_all
        self.true_value = true_value

    def __true_range(self, data_all) -> list:

        # listing the absolute value of the each all_data.
        # closing data comes from the day before.
        tr = [abs(data_all[0][0] - data_all[1][0])]
        for i in range(1, len(data_all[2])):
            tr.append(
                max(
                    abs(data_all[2][i - 1] - data_all[0][i]),
                    abs(data_all[2][i - 1] - data_all[1][i]),
                    abs(data_all[0][i] - data_all[1][i]),
                )
            )

        return tr

    def __average_true_range(self, data_all) -> list:
        # Listing the avarage true range.
        atr = [np.mean(self.true_range()[: self.true_value])]
        for i in range(len(data_all[2]) - self.true_value):
            atr.append(
                (
                    atr[i] * (self.true_value - 1)
                    + self.true_range()[i + self.true_value]
                )
                / self.true_value
            )

        return atr

    def true_range(self) -> list:
        return self.__true_range(self.data_all)

    def average_true_range(self) -> list:
        return self.__average_true_range(self.data_all)



import pandas as pd


class CMF:

    """
    This class calculate the cmf for default 20-days period.
    ...
    Attributes
    ----------
    close : pandas.Series
    low   : pandas.Series
    high  : pandas.Series
    volume: pandas.Series
    period: pandas.Series

    Methods
    -------
    money_flow_multiplier(close,low,high):
        Returns the mfm.

    money_flow_volume(mfm, volume):
        Returns the mfv.

    cmf(money_flow_multiplier,money_flow_volume,data):
        Returns the cmf.

    """

    def __init__(
        self,
        close: pd.Series,
        low: pd.Series,
        high: pd.Series,
        volume: pd.Series,
        period: int = 20,
    ):
        self.close = close
        self.low = low
        self.high = high
        self.volume = volume
        self.period = period

    def money_flow_multiplier(self) -> pd.Series:
        mfm = ((self.close - self.low) - (self.high - self.close)) / (
            self.high - self.low
        )
        try:
            self.low / self.high
        except ZeroDivisionError as e:
            print("Value of high is zero:", e)

        return mfm

    def money_flow_volume(self) -> pd.Series:
        mfv = self.money_flow_multiplier() * self.volume
        return mfv

    def cmf(self) -> pd.Series:
        volume = self.volume
        period = self.period

        mfv = self.money_flow_volume()
        cmf = mfv.rolling(window=period).sum() / volume.rolling(window=period).sum()
        return cmf




import pandas as pd

class StochasticOscillator:
    def __init__(self, high, low, close, periods: int = 5):
        self.high = high
        self.low = low
        self.close = close
        self.periods = periods
        self.df = pd.DataFrame(
            {"High": self.high, "Low": self.low, "Close": self.close})
        self.high_roll = self.highroll()
        self.low_roll = self.lowroll()
        self.df["%K"] = self.fast_stochastic()
        self.df["%D"] = self.slow_stochastic()

    def highroll(self):
        self.high_roll = self.df["High"].rolling(self.periods).max()
        return self.high_roll

    def lowroll(self):
        self.low_roll = self.df["Low"].rolling(self.periods).min()
        return self.low_roll

    # Fast stochastic indicator

    def fast_stochastic(self):
        num = self.df["Close"] - self.low_roll
        denom = self.high_roll - self.low_roll
        return (num / denom) * 100

    # Slow stochastic indicator
    def slow_stochastic(self):
        return self.df["%K"].rolling(3).mean()

    def stochasticoscillator(self):
        self.fast_stochastic()
        self.slow_stochastic()
        return self.df



from hyta.ema import EMA
import pandas as pd


class DEMA:
    """
    DEMA is a technical indicator that calculates the average
    price of an asset, giving more weight to recent prices.
    It's more responsive to price changes than a simple moving average,
    and is used by traders to identify buying and selling opportunities.
    """

    def __init__(self, close, period=9):
        self.period = period
        self.df = pd.DataFrame(data={"Close": close})
        self.close = self.df["Close"]

    def ema(self) -> pd.Series:
        """
        Calculating first EMA for DEMA
        """
        return EMA(self.close, self.period).calc_ema()

    def dema(self) -> pd.Series:
        """
        Calculating DEMA by using function --> 2*Ema - Ema of Ema
        """
        return (2 * self.ema()) - (EMA(self.ema(), self.period).calc_ema())



from cmath import nan
import numpy as np


class MA:
    '''
    This class calculates MA for default 3-days period.
    Attributes
    --------------- 
    close : list
    period : int
    '''
    def __init__(self,dataset:list,period:int=3):
        self.dataset = dataset
        self.period = period
    
    
    # Calculating the mean
    # for a given period (Default: 3 days)

    def calcAverage(self):                             
        ret = np.cumsum(self.dataset,dtype=float)      
        ret[self.period:] = ret[self.period:] - ret[:-self.period]
        return ret[self.period-1:] / self.period


     # Inserting the NaNs to the list
     # by using this function.    
    
    def nanSeries(self):
        nanSeri=[]
        for _ in range(self.period):                      
           nanSeri.append(nan)                           
        nanSeri[(self.period-1):] = self.calcAverage()
        return nanSeri

    def ma(self) -> list:
        return self.nanSeries()  




class Momentum:
    """
    Since the Momentum oscillator does not have an upper and lower boundary
    you must visually inspect the history of the momentum line and draw horizontal lines along its upper and lower boundaries.
    When the momentum line reaches these levels it may indicate that the stock may be overbought or oversold.
    The result is an indicator that oscillates around 100. Values less than 100 indicate negative momentum, or decreasing price, and vice versa.
    """

    def __init__(self, close_price_list, period: int):
        self.close_price_list = close_price_list
        self.period = period

    def momentum(self) -> float:
        # the formula of momentum
        return (self.close_price_list[-1] / self.close_price_list[-self.period]) * 100



import pandas as pd
import numpy as np


class MACD:

    """
    This class calculates MACD for default EMA1=12, EMA2=26 and SIGNAL=9.
    But you can give different input for all.

    Attributes
    ---------------
    close : list
    EMA1  : int
    EMA1  : int
    SIGNAL: int
    """

    def __init__(self, close, EMA1=12, EMA2=26, SIGNAL=9):
        self.close = close
        self.EMA1 = EMA1
        self.EMA2 = EMA2
        self.SIGNAL = SIGNAL

        # we only need the close data

    def Ema_1(self) -> list:

        Ema_first = [np.mean(self.close[: self.EMA1])]

        # Ema_12 is essential for the MACD, this "12" can be changable, but sorurces take "12".
        # For Ema_12 first we need to mean of first 12 days.
        # Other values of Ema_12 comes from the below calculation. Ema_12 values listed in "for" loop.
        for i in range(len(self.close) - self.EMA1):
            Ema_first.append(
                self.close[i + self.EMA1] * (2 / (self.EMA1 + 1))
                + (Ema_first[i] * (1 - (2 / (self.EMA1 + 1))))
            )

        return Ema_first

    def Ema_2(self) -> list:

        Ema_second = [np.mean(self.close[: self.EMA2])]

        # Ema_12 is essential for the MACD, this "12" can be changable, but sorurces take "12".
        # For Ema_12 first we need to mean of first 12 days.
        # Other values of Ema_12 comes from the below calculation. Ema_12 values listed in "for" loop.
        for i in range(len(self.close) - self.EMA2):
            Ema_second.append(
                self.close[i + self.EMA2] * (2 / (self.EMA2 + 1))
                + (Ema_second[i] * (1 - (2 / (self.EMA2 + 1))))
            )

        return Ema_second

    def MovingAverageConvergenceDivergence(self) -> list:

        # MACD values calculated with (Ema_12 - Ema_26).
        # Datas must be taken from the same day. Because of that i add "14" for Ema_12 values.
        macd = []
        for i in range(len(self.close) - self.EMA2 + 1):
            macd.append(self.Ema_1()[self.EMA2 - self.EMA1 + i] - self.Ema_2()[i])

        return macd

    def Signal(self) -> list:

        # For Signal values, first value is the mean of first 9 MACD values.
        # Other values of Signal comes from the below calculation.
        signal = [np.mean(self.MovingAverageConvergenceDivergence()[0 : self.SIGNAL])]
        for i in range(len(self.close) - self.EMA2 - self.SIGNAL + 1):
            signal.append(
                self.MovingAverageConvergenceDivergence()[i + self.SIGNAL]
                * (2 / (self.SIGNAL + 1))
                + (signal[i] * (1 - ((2 / (self.SIGNAL + 1)))))
            )

        return signal

    def Histogram(self) -> list:
        # Same day of MACD(starting from 8) and Signal values we calculate Histogram values with (MACD - Signal).
        histogram = []
        for i in range(len(self.close) - self.EMA2 - self.SIGNAL + 2):
            histogram.append(
                self.MovingAverageConvergenceDivergence()[self.SIGNAL - 1 + i]
                - self.Signal()[i]
            )
        return histogram

    def Result(self) -> pd.Series:

        d = {
            "Ema_first": pd.Series(
                self.Ema_1(),
                index=[i for i in range(0, len(self.close) - self.EMA1 + 1)],
            ),
            "Ema_second": pd.Series(
                self.Ema_2(),
                index=[
                    i
                    for i in range(
                        self.EMA2 - self.EMA1, len(self.close) - self.EMA1 + 1
                    )
                ],
            ),
            "Histogram": pd.Series(
                self.Histogram(),
                index=[
                    i
                    for i in range(
                        self.EMA2 - self.EMA1 + self.SIGNAL - 1,
                        len(self.close) - self.EMA1 + 1,
                    )
                ],
            ),
        }
        return d



import numpy as np
import pandas as pd


class DPO:
    # required input: dataset(array),period(int)
    def __init__(self, dataset: list, period: int = 4) -> None:
        self.dataset = dataset
        self.period = period

    def moving_average(self):  # Calculates the MA
        n = self.period  # for a period by using Index Slicers
        ret = np.cumsum(self.dataset)
        ret[n:] = ret[n:] - ret[:-n]
        return ret[n - 1 :] / n

    def closes(self):  # Getting (n/2)+1th close price
        n = self.period  # to calculate DPO
        tempArr = []
        for i in range(n - 2, ((n - 2) + 2 * len(self.moving_average())), 2):
            tempArr.append(self.dataset[(i // 2) + 1])
        periodCloses = np.array(tempArr)
        return periodCloses

    # Starting function, it returns a list.
    def dpo(self):
        dpoFinalize = np.subtract(self.closes(), self.moving_average())
        return pd.Series(dpoFinalize)



import pandas as pd


class PPO:
    def __init__(self, close):
        self.close = close
        self.ppo_data = pd.DataFrame({"Close": self.close})

    def ema_12(self) -> pd.Series:
        weight_p_12 = round(2 / (12 + 1), ndigits=6)

        first = self.ppo_data["Close"][0]
        ema_12_ilk = (self.ppo_data["Close"][1] * weight_p_12) + (
            first * (1 - weight_p_12)
        )

        self.ppo_data["Ema_12"] = 0
        self.ppo_data.loc[:2, "Ema_12"] = pd.Series([first, ema_12_ilk])

        for i in range(1, len(self.ppo_data["Close"])):
            ema_12_ilk = (self.ppo_data["Close"][i] * weight_p_12) + (
                self.ppo_data["Ema_12"][i - 1] * (1 - weight_p_12)
            )
            self.ppo_data["Ema_12"][i:] = ema_12_ilk
        return pd.Series(self.ppo_data["Ema_12"])

    def ema_26(self) -> pd.Series:
        weight_p_26 = round(2 / (26 + 1), ndigits=6)

        first = self.ppo_data["Close"][0]
        ema_26_ilk = (self.ppo_data["Close"][1] * weight_p_26) + (
            first * (1 - weight_p_26)
        )

        self.ppo_data["Ema_26"] = 0
        self.ppo_data.loc[:2, "Ema_26"] = pd.Series([first, ema_26_ilk])

        for i in range(1, len(self.ppo_data["Close"])):
            ema_26_ilk = (self.ppo_data["Close"][i] * weight_p_26) + (
                self.ppo_data["Ema_26"][i - 1] * (1 - weight_p_26)
            )
            self.ppo_data["Ema_26"][i:] = ema_26_ilk
        return pd.Series(self.ppo_data["Ema_26"])

    def ppo(self) -> pd.Series:
        self.ema_12()
        self.ema_26()
        self.ppo_data["PPO"] = (
            (self.ppo_data["Ema_12"] - self.ppo_data["Ema_26"])
            / self.ppo_data["Ema_12"]
        ) * 100
        return pd.Series(self.ppo_data["PPO"])



class OnBalanceVolume:
    """On Balance Volume is calculated by adding the day's volume to a cumulative total when the security's price
    closes up, and subtracting the day's volume when the security's price closes down.

    If today's close is greater than yesterday's close then:
    OBV = Yesterday’s OBV + Today’s Volume

    If today’s close is less than yesterday’s close then:
    OBV = Yesterday’s OBV - Today’s Volume

    If today’s close is equal to yesterday’s close then:
    OBV = Yesterday’s OBV
    """

    def __init__(self, close_list, volume_list, period):
        self.close_list = close_list
        self.volume_list = volume_list
        self.period = period

    def on_balance_volume(self):
        main_obv = self.volume_list[-self.period]
        # first day OBV is first day value
        for main_period in range(self.period - 1, 0, -1):
            if self.close_list[-main_period] > self.close_list[-main_period - 1]:
                main_obv += self.volume_list[-main_period]

            elif self.close_list[-main_period] < self.close_list[-main_period - 1]:
                main_obv -= self.volume_list[-main_period]

            elif self.close_list[-main_period] == self.close_list[-main_period - 1]:
                main_obv += 0

        return main_obv



import pandas as pd


class Bollinger:
    """
    Class for calculating bollinger bands
    """

    def __init__(self, df, close):
        self.df = df
        self.close = close

    def bands(self):
        df = pd.DataFrame()

        # Simple moving average and standart deviation
        # based on closing values of the last 20 days.
        df["SMA"] = self.close.rolling(window=20).mean()
        df["stddev"] = self.close.rolling(window=20).std()

        df["Upper"] = df["SMA"] + 2 * df["stddev"]
        df["Lower"] = df["SMA"] - 2 * df["stddev"]

        return df["Upper"], df["Lower"]



import numpy as np
import pandas as pd


class ROC:
    """
    This class calculate ROC for default 12-periods.
    It needs to get 'close' values respectively.
    """

    def __init__(self, close, period: int = 12) -> None:
        self.close = close
        self.period = period

    def roc(self) -> np.ndarray:
        """
        ROC = [(Close - Close n periods ago) / (Close n periods ago)] * 100
        Returns ROC array without nan values.
        """
        ROC = np.array([])
        for i in range(len(self.close) - self.period):
            ROC = np.append(
                ROC,
                ((self.close[i + self.period] - self.close[i]) / self.close[i]) * 100,
            )
        return pd.Series(ROC)



import pandas as pd
import numpy as np


class CCI:
    def __init__(self, high, low, close, period):
        self.df = pd.DataFrame(data={"high": high, "low": low, "close": close})
        self.period = period

    # typical_price as TP
    def TP(self) -> pd.Series:
        df = pd.DataFrame()
        df["TP"] = (self.df["high"] + self.df["low"] + self.df["close"]) / 3
        return df["TP"]

    # Simple_Moving_Average as SMA. It is calculated through TP.
    def SMA(self, tp: pd.Series) -> pd.Series:
        df = pd.DataFrame()
        df["TP"] = tp.values
        df["SMA"] = df["TP"].rolling(20).mean()
        return df["SMA"]

    # Mean_Deviation as MAD. It is calculated through TP.
    def MAD(self, tp: pd.Series) -> pd.Series:
        df = pd.DataFrame()
        df["TP"] = tp.values
        df["MAD"] = (
            df["TP"].rolling(20).apply(lambda x: np.mean(np.absolute(x - np.mean(x))))
        )
        return df["MAD"]

    # Commodity_Channel_Index as CCI
    def CCI(self):
        df = pd.DataFrame()
        df["TP"] = self.TP()
        df["SMA"] = self.SMA(df["TP"])
        df["MAD"] = self.MAD(df["TP"])
        df["CCI"] = (df["TP"] - df["SMA"]) / (0.015 * df["MAD"])
        return df["CCI"]



class WeightedMovingAverage:
    """
    A Weighted Moving Average puts more weight on recent data and less on past data.
    This is done by multiplying each bar’s price by a weighting factor.
    """

    def __init__(self, close_list: list, period: int):
        self.close_list = close_list
        self.period = period

    def weighted_moving_average(self) -> float:
        # the formula of wma
        # P1 = current close price, Pn = n days ago close price
        # wma = P1*n + P2*(n-1) + ..... + Pn*1 / n + (n-1) + ..... +1
        top_wma = 0
        bottom_wma = 0
        for i in range(1, self.period + 1):
            top_wma += self.close_list[-i] * (self.period + 1 - i)
            bottom_wma += i

        return top_wma / bottom_wma



import pandas as pd
import numpy as np
from hyta.dx import DX


class ADX:
    def __init__(self, high, low, close, period=14):
        self.data = pd.DataFrame({"high": high, "low": low, "close": close})
        self.period = period

    def adx(self):
        dx_data = DX(self.data["high"], self.data["low"], self.data["close"]).dx()
        dx_mean = dx_data[self.period : (self.period + 13)].mean()

        self.data["ADX"] = np.nan
        self.data["ADX"][27] = dx_mean
        other_value = []
        for i in range(28, len(dx_data) + 1):
            adx_result = (
                (self.data["ADX"][i - 1] * (self.period - 1)) + dx_data[i - 1]
            ) / self.period
            other_value.append(adx_result)
            self.data["ADX"][28:] = other_value[i - 28]

        return other_value[-1]


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='hyta',
    version='0.1.4',
    author='Toygar Aksoy',
    author_email='toygar.aksoy@gmail.com',
    description='Indicators for Hypance Project',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/Hypance/HypanceDataAnalysis',
    # project_urls = {
    #     "Bug Tracker": "https://github.com/Hypance/HypanceDataAnalysis/issues"
    # },
    license='MIT',
    packages=['hyta'],
    install_requires=['pandas', 'numpy'],
)
